@@ -9,12 +9,12 @@ import numpy as np
 import xarray as xr
 from qtpy import QtCore, QtWidgets
 import SimpleITK as sitk
-from scipy import ndimage
+from scipy import ndimage, stats
 import cv2
 from stardist.models import StarDist2D
 from napari_animation import AnimationWidget
 from PIL import Image, ImageDraw
-
+import random
 # import skimage.io
 # from stardist.models import StarDist3D
 from csbdeep.utils import normalize
@@ -23,11 +23,13 @@ import math
 import gc
 import json
 
+
 class NapariSTPT:
 
     def __init__(self):
         self.scroll = None
         self.image_slice = None
+        self.normalize_value = None
 
         self.pixel_size = None
         self.viewer = None
@@ -54,12 +56,17 @@ class NapariSTPT:
         self.corrected_align_y = None
         self.ds1 = None
         self.im1 = None
+        self.im2 = None
+        self.im4 = None
+        self.im8 = None
+        self.im16 = None
         self.im32 = None
 
         #self.movie = None
         self.optical_slices = None
         self.nr_optical_slices = None
         self.optical_slices_available = None
+        self.cb_correct_brightness_optical_section = None
         self.shape = None
 
         self.crop_start_ratio_x = None
@@ -305,13 +312,16 @@ class NapariSTPT:
                     volume_1[optical_slice::self.optical_slices, :, :] = volume_1_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 # self.aligned_2 = volume_2
+                if self.cb_correct_brightness_optical_section.isChecked():
+                    print("correcting brightness of optical sections C2")
+                    self.Normalize_slices(volume_1, self.optical_slices)
 
                 print("aligning C1")
                 self.aligned_1 = self.Align(volume_1, resolution, output_resolution, start_slice_number*self.optical_slices)
                 #self.aligned_1 = volume_1
                 self.aligned_1 = self.aligned_1[chop_bottom:self.aligned_1.shape[0]-chop_top,:,:]
                 self.shape = self.aligned_1.shape
-                self.viewer.add_image([self.aligned_1], name='C1', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop purple')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                self.viewer.add_image([self.aligned_1], name='C1', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop purple', contrast_limits=[0,30])#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             if self.cb_C2.isChecked():
                 print("loading C2")
@@ -358,13 +368,17 @@ class NapariSTPT:
                     volume_2[optical_slice::self.optical_slices, :, :] = volume_2_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 # self.aligned_2 = volume_2
+                if self.cb_correct_brightness_optical_section.isChecked():
+                    print("correcting brightness of optical sections C2")
+                    self.Normalize_slices(volume_2, self.optical_slices)
+
 
                 print("aligning C2")
                 self.aligned_2 = self.Align(volume_2, resolution, output_resolution, start_slice_number*self.optical_slices)
                 #self.aligned_2 = volume_2
                 self.aligned_2 = self.aligned_2[chop_bottom:self.aligned_2.shape[0]-chop_top,:,:]
                 self.shape = self.aligned_2.shape
-                self.viewer.add_image([self.aligned_2], name='C2', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='red')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                self.viewer.add_image([self.aligned_2], name='C2', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='red', contrast_limits=[0,30])#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             if self.cb_C3.isChecked():
                 print("loading C3")
@@ -411,13 +425,16 @@ class NapariSTPT:
                     volume_3[optical_slice::self.optical_slices, :, :] = volume_3_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 # self.aligned_3 = volume_3
+                if self.cb_correct_brightness_optical_section.isChecked():
+                    print("correcting brightness of optical sections C2")
+                    self.Normalize_slices(volume_3, self.optical_slices)
 
                 print("aligning C3")
                 self.aligned_3 = self.Align(volume_3, resolution, output_resolution, start_slice_number*self.optical_slices)
                 #self.aligned_3 = volume_3
                 self.aligned_3 = self.aligned_3[chop_bottom:self.aligned_3.shape[0]-chop_top,:,:]
                 self.shape = self.aligned_3.shape
-                self.viewer.add_image([self.aligned_3], name='C3', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='green')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                self.viewer.add_image([self.aligned_3], name='C3', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='green', contrast_limits=[0,30])#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             if self.cb_C4.isChecked():
                 print("loading C4")
@@ -464,13 +481,16 @@ class NapariSTPT:
                     volume_4[optical_slice::self.optical_slices, :, :] = volume_4_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 # self.aligned_4 = volume_4
+                if self.cb_correct_brightness_optical_section.isChecked():
+                    print("correcting brightness of optical sections C2")
+                    self.Normalize_slices(volume_4, self.optical_slices)
 
                 print("aligning C4")
                 self.aligned_4 = self.Align(volume_4, resolution, output_resolution, start_slice_number*self.optical_slices)
                 #self.aligned_4 = volume_4
                 self.aligned_4 = self.aligned_4[chop_bottom:self.aligned_4.shape[0]-chop_top,:,:]
                 self.shape = self.aligned_4.shape
-                self.viewer.add_image([self.aligned_4], name='C4', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop blue')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                self.viewer.add_image([self.aligned_4], name='C4', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop blue', contrast_limits=[0,30])#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             # create naparimovie object
             #if self.movie is None:
@@ -543,19 +563,29 @@ class NapariSTPT:
         ds32 = xr.open_zarr(self.search_folder.text(
         ) + str(self.comboBoxPath.currentText())+'/mos.zarr', group='l.32')
 
-        bscale = self.ds1.attrs['bscale']
-        bzero = self.ds1.attrs['bzero']
-
+        
         z = self.scroll.value()
 
-        self.slice_names = self.ds1.attrs['cube_reg']['slice']
+        if self.old_method:
+            #Old method
+            bscale = self.ds1.attrs['bscale']
+            bzero = self.ds1.attrs['bzero']
+
+            self.slice_names = self.ds1.attrs['cube_reg']['slice']
+
+            self.scroll.setRange(0, len(self.slice_names)-1)
+            if z >= len(self.slice_names):
+                z = len(self.slice_names)-1
+                self.scroll.setValue(z)
+
+            slice_name = self.slice_names[z]
+        else:
+            bscale = self.ds1['S001'].attrs['bscale']
+            bzero = self.ds1['S001'].attrs['bzero']
+            slice_name = f"S{(z+1):03d}"
+
 
-        self.scroll.setRange(0, len(self.slice_names)-1)
-        if z >= len(self.slice_names):
-            z = len(self.slice_names)-1
-            self.scroll.setValue(z)
 
-        slice_name = self.slice_names[z]
 
         # z = 0
         # slice_name = 'S00'+str(z+1)
@@ -564,78 +594,80 @@ class NapariSTPT:
         # if(z+1 > 99):
         #    slice_name = 'S'+str(z+1)
 
+
         if self.cb_C1.isChecked():
-            im1 = self.ds1[slice_name].sel(
-                channel=1, type='mosaic', z=0).data * bscale + bzero
-            im2 = ds2[slice_name].sel(
-                channel=1, type='mosaic', z=0).data * bscale + bzero
-            im4 = ds4[slice_name].sel(
-                channel=1, type='mosaic', z=0).data * bscale + bzero
-            im8 = ds8[slice_name].sel(
-                channel=1, type='mosaic', z=0).data * bscale + bzero
-            im16 = ds16[slice_name].sel(
-                channel=1, type='mosaic', z=0).data * bscale + bzero
-            self.im32 = ds32[slice_name].sel(
-                channel=1, type='mosaic', z=0).data * bscale + bzero
-
-            with napari.gui_qt():
-                self.viewer.add_image([im1, im2, im4, im8, im16, self.im32], multiscale=True,
-                                      name='C1', blending='additive', colormap='bop purple')
-
-        if self.cb_C2.isChecked():
-            im1 = self.ds1[slice_name].sel(
-                channel=2, type='mosaic', z=0).data * bscale + bzero
-            im2 = ds2[slice_name].sel(
-                channel=2, type='mosaic', z=0).data * bscale + bzero
-            im4 = ds4[slice_name].sel(
-                channel=2, type='mosaic', z=0).data * bscale + bzero
-            im8 = ds8[slice_name].sel(
-                channel=2, type='mosaic', z=0).data * bscale + bzero
-            im16 = ds16[slice_name].sel(
-                channel=2, type='mosaic', z=0).data * bscale + bzero
-            im32 = ds32[slice_name].sel(
-                channel=2, type='mosaic', z=0).data * bscale + bzero
-
-            with napari.gui_qt():
-                self.viewer.add_image([im1, im2, im4, im8, im16, im32],
-                                      multiscale=True, name='C2', blending='additive', colormap='red')
-
-        if self.cb_C3.isChecked():
-            im1 = self.ds1[slice_name].sel(
-                channel=3, type='mosaic', z=0).data * bscale + bzero
-            im2 = ds2[slice_name].sel(
-                channel=3, type='mosaic', z=0).data * bscale + bzero
-            im4 = ds4[slice_name].sel(
-                channel=3, type='mosaic', z=0).data * bscale + bzero
-            im8 = ds8[slice_name].sel(
-                channel=3, type='mosaic', z=0).data * bscale + bzero
-            im16 = ds16[slice_name].sel(
-                channel=3, type='mosaic', z=0).data * bscale + bzero
-            im32 = ds32[slice_name].sel(
-                channel=3, type='mosaic', z=0).data * bscale + bzero
+            im1 = (self.ds1[slice_name].sel(
+                channel=1, type='mosaic', z=0).data * bscale + bzero)
+            im2 = (ds2[slice_name].sel(
+                channel=1, type='mosaic', z=0).data * bscale + bzero)
+            im4 = (ds4[slice_name].sel(
+                channel=1, type='mosaic', z=0).data * bscale + bzero)
+            im8 = (ds8[slice_name].sel(
+                channel=1, type='mosaic', z=0).data * bscale + bzero)
+            im16 = (ds16[slice_name].sel(
+                channel=1, type='mosaic', z=0).data * bscale + bzero)
+            im32 = (ds32[slice_name].sel(
+                channel=1, type='mosaic', z=0).data * bscale + bzero)
 
             with napari.gui_qt():
                 self.viewer.add_image([im1, im2, im4, im8, im16, im32], multiscale=True,
-                                      name='C3', blending='additive', colormap='green')
+                                      name='C1', blending='additive', colormap='bop purple', contrast_limits=[0,30])
+
+        if self.cb_C2.isChecked():
+            im1 = (self.ds1[slice_name].sel(
+                channel=2, type='mosaic', z=0).data * bscale + bzero)
+            im2 = (ds2[slice_name].sel(
+                channel=2, type='mosaic', z=0).data * bscale + bzero)
+            im4 = (ds4[slice_name].sel(
+                channel=2, type='mosaic', z=0).data * bscale + bzero)
+            im8 = (ds8[slice_name].sel(
+                channel=2, type='mosaic', z=0).data * bscale + bzero)
+            im16 = (ds16[slice_name].sel(
+                channel=2, type='mosaic', z=0).data * bscale + bzero)
+            im32 = (ds32[slice_name].sel(
+                channel=2, type='mosaic', z=0).data * bscale + bzero)
+
+            with napari.gui_qt():
+                self.viewer.add_image([im1, im2, im4, im8, im16, im32], multiscale=True, 
+                                        name='C2', blending='additive', colormap='red', contrast_limits=[0,30])
+                
+
+        if self.cb_C3.isChecked():
+            im1 = (self.ds1[slice_name].sel(
+                channel=3, type='mosaic', z=0).data * bscale + bzero)
+            im2 = (ds2[slice_name].sel(
+                channel=3, type='mosaic', z=0).data * bscale + bzero)
+            im4 = (ds4[slice_name].sel(
+                channel=3, type='mosaic', z=0).data * bscale + bzero)
+            im8 = (ds8[slice_name].sel(
+                channel=3, type='mosaic', z=0).data * bscale + bzero)
+            im16 = (ds16[slice_name].sel(
+                channel=3, type='mosaic', z=0).data * bscale + bzero)
+            im32 = (ds32[slice_name].sel(
+                channel=3, type='mosaic', z=0).data * bscale + bzero)
+
+            with napari.gui_qt():
+                self.viewer.add_image([im1, im2, im4, im8, im16, im32], multiscale=True,
+                                      name='C3', blending='additive', colormap='green', contrast_limits=[0,30])
                 #self.viewer.add_image(im4, multiscale=False, name='C3', blending='additive', colormap='green')
 
         if self.cb_C4.isChecked():
-            im1 = self.ds1[slice_name].sel(
-                channel=4, type='mosaic', z=0).data * bscale + bzero
-            im2 = ds2[slice_name].sel(
-                channel=4, type='mosaic', z=0).data * bscale + bzero
-            im4 = ds4[slice_name].sel(
-                channel=4, type='mosaic', z=0).data * bscale + bzero
-            im8 = ds8[slice_name].sel(
-                channel=4, type='mosaic', z=0).data * bscale + bzero
-            im16 = ds16[slice_name].sel(
-                channel=4, type='mosaic', z=0).data * bscale + bzero
-            im32 = ds32[slice_name].sel(
-                channel=4, type='mosaic', z=0).data * bscale + bzero
+            im1 = (self.ds1[slice_name].sel(
+                channel=4, type='mosaic', z=0).data * bscale + bzero)
+            im2 = (ds2[slice_name].sel(
+                channel=4, type='mosaic', z=0).data * bscale + bzero)
+            im4 = (ds4[slice_name].sel(
+                channel=4, type='mosaic', z=0).data * bscale + bzero)
+            im8 = (ds8[slice_name].sel(
+                channel=4, type='mosaic', z=0).data * bscale + bzero)
+            im16 = (ds16[slice_name].sel(
+                channel=4, type='mosaic', z=0).data * bscale + bzero)
+            im32 = (ds32[slice_name].sel(
+                channel=4, type='mosaic', z=0).data * bscale + bzero)
 
             with napari.gui_qt():
                 self.viewer.add_image([im1, im2, im4, im8, im16, im32], multiscale=True,
-                                      name='C4', blending='additive', colormap='bop blue')
+                                      name='C4', blending='additive', colormap='bop blue', contrast_limits=[0,30])
 
 
     def Align(self, volume, resolution, output_resolution, start_slice_number):
@@ -1059,25 +1091,25 @@ class NapariSTPT:
                                             minY_final:maxY_final, minZ_final:maxZ_final]
             self.viewer.layers.remove('C1')
             self.viewer.add_image([self.aligned_1], name='C1', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='bop purple')
+                15, output_resolution, output_resolution), blending='additive', colormap='bop purple', contrast_limits=[0,30])
         if self.aligned_2 is not None and self.cb_C2.isChecked() and any(i.name == 'C2' for i in self.viewer.layers):
             self.aligned_2 = self.aligned_2[minX_final:maxX_final,
                                             minY_final:maxY_final, minZ_final:maxZ_final]
             self.viewer.layers.remove('C2')
             self.viewer.add_image([self.aligned_2], name='C2', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='red')
+                15, output_resolution, output_resolution), blending='additive', colormap='red', contrast_limits=[0,30])
         if self.aligned_3 is not None and self.cb_C3.isChecked() and any(i.name == 'C3' for i in self.viewer.layers):
             self.aligned_3 = self.aligned_3[minX_final:maxX_final,
                                             minY_final:maxY_final, minZ_final:maxZ_final]
             self.viewer.layers.remove('C3')
             self.viewer.add_image([self.aligned_3], name='C3', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='green')
+                15, output_resolution, output_resolution), blending='additive', colormap='green', contrast_limits=[0,30])
         if self.aligned_4 is not None and self.cb_C4.isChecked() and any(i.name == 'C4' for i in self.viewer.layers):
             self.aligned_4 = self.aligned_4[minX_final:maxX_final,
                                             minY_final:maxY_final, minZ_final:maxZ_final]
             self.viewer.layers.remove('C4')
             self.viewer.add_image([self.aligned_4], name='C4', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='bop blue')
+                15, output_resolution, output_resolution), blending='additive', colormap='bop blue', contrast_limits=[0,30])
 
         self.viewer.layers.remove('Shapes')
         
@@ -1670,69 +1702,154 @@ class NapariSTPT:
             sitk.WriteImage(image_itk, fname[0])
 
 
-    def Normalize_slices(self):
+    def Normalize_slices(self, volume, optical_sections):
         import statistics
-        print("Normalize_slices")
-        
+        print("Normalize optical sections")
 
-        output_resolution = float(self.pixel_size.text())
+        #output_resolution = float(self.pixel_size.text())
 
-        if self.cb_C1.isChecked():
+        #self.aligned_3 = self.aligned_3[4::5,:,:]
+
+        #with napari.gui_qt():
+        #    self.viewer.layers.remove('C3')
+        #    self.viewer.add_image([self.aligned_3], name='C3', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='green', contrast_limits=[0,30])#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+
+        #return 
+
+
+        #output_resolution = float(self.pixel_size.text())
+
+        #if self.cb_C2.isChecked():
+
+
+        slices, size_x, size_y = volume.shape
+        for i in range(1,slices):
+            if i%optical_sections != 0:
+                values_x = []
+                values_y = []
+                for j in range(1000):
+                    rand_x = random.randint(1,size_x-1)
+                    rand_y = random.randint(1,size_y-1)
+                    if(volume[i-1,rand_x,rand_y] > 0.0 and volume[i,rand_x,rand_y] > 0.0):
+                        x = volume[i,rand_x,rand_y]
+                        y = volume[i-1,rand_x,rand_y]
+                        values_x.append(x)
+                        values_y.append(y)
+
+                slope, intercept, r, p, std_err = stats.linregress(values_x, values_y)
+
+                volume[i,:,:] = volume[i,:,:] * slope + intercept
+
+        return
+
+
+
+
+        slices, size_x, size_y = volume.shape
+        for i in range(1,slices):
+            if i%5 != 0:
+                values = []
+                for j in range(10000):
+                    rand_x = random.randint(1,size_x-1)
+                    rand_y = random.randint(1,size_y-1)
+                    if(volume[i-1,rand_x,rand_y] > 0.1 and volume[i,rand_x,rand_y] > 0.1 and volume[i-1,rand_x,rand_y] < 1 and volume[i,rand_x,rand_y] < 1):
+                        division = volume[i-1,rand_x,rand_y] / volume[i,rand_x,rand_y]
+                        values.append(division)
+
+                mean = sum(values) / len(values)
+                volume[i,:,:] = volume[i,:,:] * mean
+
+        return
+
+        if self.cb_C3.isChecked():  
             slices, size_x, size_y = self.aligned_3.shape
-            for i in range(0,slices):
+            for i in range(1,slices):
+                if i%5 != 0:
+                    values = []
+                    for j in range(1000):
+                        rand_x = random.randint(1,size_x-1)
+                        rand_y = random.randint(1,size_y-1)
+                        if(self.aligned_3[i-1,rand_x,rand_y] > 0.1 and self.aligned_3[i,rand_x,rand_y] > 0.1):
+                            division = self.aligned_3[i-1,rand_x,rand_y] / self.aligned_3[i,rand_x,rand_y]
+                            values.append(division)
 
-                mean = np.mean(self.aligned_1[i,:,:])
-                self.aligned_1[i,:,:] = self.aligned_1[i,:,:] - mean
-
-                st_dev = np.std(self.aligned_1[i,:,:])
-                self.aligned_1[i,:,:] = self.aligned_1[i,:,:] / st_dev
-
-            with napari.gui_qt():
-                self.viewer.layers.remove('C1')
-                self.viewer.add_image([self.aligned_1], name='C1', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop purple')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
-
-
-        if self.cb_C2.isChecked():  
-            slices, size_x, size_y = self.aligned_3.shape
-            for i in range(0,slices):
-
-                mean = np.mean(self.aligned_2[i,:,:])
-                self.aligned_2[i,:,:] = self.aligned_2[i,:,:] - mean
-
-                st_dev = np.std(self.aligned_2[i,:,:])
-                self.aligned_2[i,:,:] = self.aligned_2[i,:,:] / st_dev
-
-            with napari.gui_qt():
-                self.viewer.layers.remove('C2')
-                self.viewer.add_image([self.aligned_2], name='C2', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='red')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
-
-        if self.cb_C3.isChecked():
-            slices, size_x, size_y = self.aligned_3.shape
-            for i in range(0,slices):
-
-                mean = np.mean(self.aligned_3[i,:,:])
-                self.aligned_3[i,:,:] = self.aligned_3[i,:,:] - mean
-
-                st_dev = np.std(self.aligned_3[i,:,:])
-                self.aligned_3[i,:,:] = self.aligned_3[i,:,:] / st_dev
+                    mean = sum(values) / len(values)
+                    self.aligned_3[i,:,:] = self.aligned_3[i,:,:] * mean
 
             with napari.gui_qt():
                 self.viewer.layers.remove('C3')
-                self.viewer.add_image([self.aligned_3], name='C3', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='green')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                self.viewer.add_image([self.aligned_3], name='C3', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='green', contrast_limits=[0,30])#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+
+
+
+        return
+
+
+
+    def Normalize(self):
+
+        output_resolution = float(self.pixel_size.text())
+        norm_value = float(self.normalize_value.text())
+
+        if self.cb_C1.isChecked():
+            self.aligned_1[0::10,:,:] = self.aligned_1[0::10,:,:] * norm_value
+            self.aligned_1[1::10,:,:] = self.aligned_1[1::10,:,:] * norm_value
+            self.aligned_1[2::10,:,:] = self.aligned_1[2::10,:,:] * norm_value
+            self.aligned_1[3::10,:,:] = self.aligned_1[3::10,:,:] * norm_value
+            self.aligned_1[4::10,:,:] = self.aligned_1[4::10,:,:] * norm_value
+
+            with napari.gui_qt():
+                self.viewer.layers.remove('C1')
+                self.viewer.add_image([self.aligned_1], name='C1', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop purple', contrast_limits=[0,30])#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+
+
+        if self.cb_C2.isChecked():  
+            self.aligned_2[0::10,:,:] = self.aligned_2[0::10,:,:] * norm_value
+            self.aligned_2[1::10,:,:] = self.aligned_2[1::10,:,:] * norm_value
+            self.aligned_2[2::10,:,:] = self.aligned_2[2::10,:,:] * norm_value
+            self.aligned_2[3::10,:,:] = self.aligned_2[3::10,:,:] * norm_value
+            self.aligned_2[4::10,:,:] = self.aligned_2[4::10,:,:] * norm_value
+
+            with napari.gui_qt():
+                self.viewer.layers.remove('C2')
+                self.viewer.add_image([self.aligned_2], name='C2', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='red', contrast_limits=[0,30])#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+
+        if self.cb_C3.isChecked():
+
+            slices, size_x, size_y = self.aligned_3.shape
+            for i in range(1,slices):
+                if i%2 != 0:
+                    values_x = []
+                    values_y = []
+                    for j in range(1000):
+                        rand_x = random.randint(1,size_x-1)
+                        rand_y = random.randint(1,size_y-1)
+                        if(self.aligned_3[i-1,rand_x,rand_y] > 0.0 and self.aligned_3[i,rand_x,rand_y] > 0.0):
+                            x = self.aligned_3[i,rand_x,rand_y]
+                            y = self.aligned_3[i-1,rand_x,rand_y]
+                            values_x.append(x)
+                            values_y.append(y)
+
+                    slope, intercept, r, p, std_err = stats.linregress(values_x, values_y)
+
+                    print(slope)
+
+                    self.aligned_3[i,:,:] = self.aligned_3[i,:,:] * slope + intercept
+
+            with napari.gui_qt():
+                self.viewer.layers.remove('C3')
+                self.viewer.add_image([self.aligned_3], name='C3', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='green', contrast_limits=[0,30])#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
         if self.cb_C4.isChecked():
-            slices, size_x, size_y = self.aligned_4.shape
-            for i in range(0,slices):
-
-                mean = np.mean(self.aligned_4[i,:,:])
-                self.aligned_4[i,:,:] = self.aligned_4[i,:,:] - mean
-
-                st_dev = np.std(self.aligned_4[i,:,:])
-                self.aligned_4[i,:,:] = self.aligned_4[i,:,:] / st_dev
+            self.aligned_4[0::10,:,:] = self.aligned_4[0::10,:,:] * norm_value
+            self.aligned_4[1::10,:,:] = self.aligned_4[1::10,:,:] * norm_value
+            self.aligned_4[2::10,:,:] = self.aligned_4[2::10,:,:] * norm_value
+            self.aligned_4[3::10,:,:] = self.aligned_4[3::10,:,:] * norm_value
+            self.aligned_4[4::10,:,:] = self.aligned_4[4::10,:,:] * norm_value
 
             with napari.gui_qt():
                 self.viewer.layers.remove('C4')
-                self.viewer.add_image([self.aligned_4], name='C4', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop blue')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                self.viewer.add_image([self.aligned_4], name='C4', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop blue', contrast_limits=[0,30])#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             
 
@@ -1768,6 +1885,8 @@ class NapariSTPT:
             hbox = QtWidgets.QHBoxLayout()
             self.comboBoxPath = QtWidgets.QComboBox()
 
+            #text_file = open("sample.txt", "wt")
+
             self.comboBoxPath.clear()
             if os.path.exists(self.search_folder.text()):
                 for f in os.scandir(self.search_folder.text()):
@@ -1776,8 +1895,13 @@ class NapariSTPT:
                             s = f.path
                             s = s.replace(self.search_folder.text(), '')
                             print(s)
+                            # print(type(s))
+                            # text_file.write(s)
+                            # text_file.write('\n')
                             self.comboBoxPath.addItem(s)
 
+            # text_file.close()
+
             self.comboBoxPath.setMaximumWidth(300)
             self.comboBoxPath.currentIndexChanged.connect(
                 self.on_combobox_changed)
@@ -1805,6 +1929,13 @@ class NapariSTPT:
             hbox.addStretch(1)
             vbox.addLayout(hbox)
 
+
+            hbox = QtWidgets.QHBoxLayout()
+            self.cb_correct_brightness_optical_section = QtWidgets.QCheckBox('Normalize brightness optical sections')
+            self.cb_correct_brightness_optical_section.setChecked(True)
+            hbox.addWidget(self.cb_correct_brightness_optical_section)
+            hbox.addStretch(1)
+            vbox.addLayout(hbox)
             
 
             hbox = QtWidgets.QHBoxLayout()
@@ -1935,10 +2066,14 @@ class NapariSTPT:
             bSaveVolume.clicked.connect(self.SaveVolume)
             hbox.addWidget(bSaveVolume)
 
-            bNormalize = QtWidgets.QPushButton('Normalize')
-            bNormalize.setCheckable(True)
-            bNormalize.clicked.connect(self.Normalize_slices)
-            hbox.addWidget(bNormalize)
+            # bNormalize = QtWidgets.QPushButton('Normalize')
+            # bNormalize.setCheckable(True)
+            # bNormalize.clicked.connect(self.Normalize)
+            # hbox.addWidget(bNormalize)
+
+            # self.normalize_value = QtWidgets.QLineEdit("0")
+            # self.normalize_value.setMaximumWidth(30)
+            # hbox.addWidget(self.normalize_value)
 
             #bSaveMovie = QtWidgets.QPushButton('Save movie')
             #bSaveMovie.setCheckable(True)
